@@ -1,7 +1,7 @@
 invMillsRatio <- function( x, all = FALSE ) {
    errorMessage <- paste( "calculating the 'Inverse Mills Ratio' only works",
       "for probit models estimated by 'glm' or 'vglm' or 'probit'." )
-   if( class( x )[ 1 ] == "glm" ) {
+   if(inherits( x, "glm") ) {
       if( x$family$family != "binomial" || x$family$link != "probit" ) {
          stop( errorMessage )
       }
@@ -13,16 +13,16 @@ invMillsRatio <- function( x, all = FALSE ) {
       result$IMR0 <- dnorm( x$linear.predictors ) /
          pnorm( -x$linear.predictors )
       result$delta0 <- result$IMR0 * ( result$IMR0 + x$linear.predictors )
-   } else if("probit" %in% class(x)) {
+   } else if(inherits(x, "probit")) {
       # Note: 'probit' need not to be the first component in the class
-      result <- data.frame( no = 1:nrow( model.frame( x ) ),
+      result <- data.frame( no = seq(length=nObs(x)),
          row.names = rownames( model.frame( x ) ) )
       result$IMR1 <- dnorm(linearPredictors(x))/pnorm(linearPredictors(x))
       result$delta1 <- result$IMR1 * ( result$IMR1 + linearPredictors(x))
       result$IMR0 <- dnorm(linearPredictors(x))/pnorm(-linearPredictors(x))
       result$delta0 <- result$IMR0 * ( result$IMR0 + linearPredictors(x))
 
-   } else if("vglm" %in% class( x )) {
+   } else if(inherits(x, "vglm")) {
       library( VGAM )
       if( x@family@blurb[1] != "Bivariate probit model\n"  ) {
          stop( errorMessage )
